@@ -12,6 +12,14 @@ pub trait ConstantFrame: Clone {
     fn convert(self) -> Frame<Self::Sample, Self::Size>;
 }
 
+impl<T: Float, N: Size<T>> ConstantFrame for Frame<T, N> {
+    type Sample = T;
+    type Size = N;
+    fn convert(self) -> Frame<Self::Sample, Self::Size> {
+        self
+    }
+}
+
 impl<T: Float> ConstantFrame for T {
     type Sample = T;
     type Size = U1;
@@ -182,7 +190,7 @@ where
     X: AudioNode,
     X::Outputs: Size<X::Sample>,
 {
-    type Output = An<Unop<X::Sample, X, FrameNeg<X::Sample, X::Outputs>>>;
+    type Output = An<Unop<X::Sample, X, FrameNeg<X::Outputs, X::Sample>>>;
     #[inline]
     fn neg(self) -> Self::Output {
         An(Unop::new(self.0, FrameNeg::new()))
@@ -212,7 +220,7 @@ where
     Y::Inputs: Size<Y::Sample>,
     <X::Inputs as Add<Y::Inputs>>::Output: Size<X::Sample>,
 {
-    type Output = An<Binop<X::Sample, FrameAdd<X::Sample, X::Outputs>, X, Y>>;
+    type Output = An<Binop<X::Sample, FrameAdd<X::Outputs, X::Sample>, X, Y>>;
     #[inline]
     fn add(self, y: An<Y>) -> Self::Output {
         An(Binop::new(self.0, y.0, FrameAdd::new()))
@@ -228,7 +236,7 @@ where
     X::Outputs: Size<f64>,
     <X::Inputs as Add<U0>>::Output: Size<f64>,
 {
-    type Output = An<Binop<f64, FrameAdd<X::Sample, X::Outputs>, X, Constant<f64, X::Outputs>>>;
+    type Output = An<Binop<f64, FrameAdd<X::Outputs, X::Sample>, X, Constant<X::Outputs, f64>>>;
     #[inline]
     fn add(self, y: f64) -> Self::Output {
         An(Binop::new(
@@ -248,7 +256,7 @@ where
     X::Outputs: Size<f64>,
     <X::Inputs as Add<U0>>::Output: Size<f64>,
 {
-    type Output = An<Binop<f64, FrameAdd<f64, X::Outputs>, Constant<f64, X::Outputs>, X>>;
+    type Output = An<Binop<f64, FrameAdd<X::Outputs, f64>, Constant<X::Outputs, f64>, X>>;
     #[inline]
     fn add(self, y: An<X>) -> Self::Output {
         An(Binop::new(
@@ -268,7 +276,7 @@ where
     X::Outputs: Size<f32>,
     <X::Inputs as Add<U0>>::Output: Size<f32>,
 {
-    type Output = An<Binop<f32, FrameAdd<X::Sample, X::Outputs>, X, Constant<f32, X::Outputs>>>;
+    type Output = An<Binop<f32, FrameAdd<X::Outputs, X::Sample>, X, Constant<X::Outputs, f32>>>;
     #[inline]
     fn add(self, y: f32) -> Self::Output {
         An(Binop::new(
@@ -288,7 +296,7 @@ where
     X::Outputs: Size<f32>,
     <X::Inputs as Add<U0>>::Output: Size<f32>,
 {
-    type Output = An<Binop<f32, FrameAdd<f32, X::Outputs>, Constant<f32, X::Outputs>, X>>;
+    type Output = An<Binop<f32, FrameAdd<X::Outputs, f32>, Constant<X::Outputs, f32>, X>>;
     #[inline]
     fn add(self, y: An<X>) -> Self::Output {
         An(Binop::new(
@@ -309,7 +317,7 @@ where
     Y::Inputs: Size<Y::Sample>,
     <X::Inputs as Add<Y::Inputs>>::Output: Size<X::Sample>,
 {
-    type Output = An<Binop<X::Sample, FrameSub<X::Sample, X::Outputs>, X, Y>>;
+    type Output = An<Binop<X::Sample, FrameSub<X::Outputs, X::Sample>, X, Y>>;
     #[inline]
     fn sub(self, y: An<Y>) -> Self::Output {
         An(Binop::new(self.0, y.0, FrameSub::new()))
@@ -325,7 +333,7 @@ where
     X::Outputs: Size<f64>,
     <X::Inputs as Add<U0>>::Output: Size<f64>,
 {
-    type Output = An<Binop<f64, FrameSub<f64, X::Outputs>, X, Constant<f64, X::Outputs>>>;
+    type Output = An<Binop<f64, FrameSub<X::Outputs, f64>, X, Constant<X::Outputs, f64>>>;
     #[inline]
     fn sub(self, y: f64) -> Self::Output {
         An(Binop::new(
@@ -345,7 +353,7 @@ where
     X::Outputs: Size<f64>,
     <X::Inputs as Add<U0>>::Output: Size<f64>,
 {
-    type Output = An<Binop<f64, FrameSub<f64, X::Outputs>, Constant<f64, X::Outputs>, X>>;
+    type Output = An<Binop<f64, FrameSub<X::Outputs, f64>, Constant<X::Outputs, f64>, X>>;
     #[inline]
     fn sub(self, y: An<X>) -> Self::Output {
         An(Binop::new(
@@ -365,7 +373,7 @@ where
     X::Outputs: Size<f32>,
     <X::Inputs as Add<U0>>::Output: Size<f32>,
 {
-    type Output = An<Binop<f32, FrameSub<f32, X::Outputs>, X, Constant<f32, X::Outputs>>>;
+    type Output = An<Binop<f32, FrameSub<X::Outputs, f32>, X, Constant<X::Outputs, f32>>>;
     #[inline]
     fn sub(self, y: f32) -> Self::Output {
         An(Binop::new(
@@ -385,7 +393,7 @@ where
     X::Outputs: Size<f32>,
     <X::Inputs as Add<U0>>::Output: Size<f32>,
 {
-    type Output = An<Binop<f32, FrameSub<f32, X::Outputs>, Constant<f32, X::Outputs>, X>>;
+    type Output = An<Binop<f32, FrameSub<X::Outputs, f32>, Constant<X::Outputs, f32>, X>>;
     #[inline]
     fn sub(self, y: An<X>) -> Self::Output {
         An(Binop::new(
@@ -406,7 +414,7 @@ where
     Y::Inputs: Size<Y::Sample>,
     <X::Inputs as Add<Y::Inputs>>::Output: Size<X::Sample>,
 {
-    type Output = An<Binop<X::Sample, FrameMul<X::Sample, X::Outputs>, X, Y>>;
+    type Output = An<Binop<X::Sample, FrameMul<X::Outputs, X::Sample>, X, Y>>;
     #[inline]
     fn mul(self, y: An<Y>) -> Self::Output {
         An(Binop::new(self.0, y.0, FrameMul::new()))
@@ -422,7 +430,7 @@ where
     X::Outputs: Size<f64>,
     <X::Inputs as Add<U0>>::Output: Size<f64>,
 {
-    type Output = An<Binop<f64, FrameMul<f64, X::Outputs>, X, Constant<f64, X::Outputs>>>;
+    type Output = An<Binop<f64, FrameMul<X::Outputs, f64>, X, Constant<X::Outputs, f64>>>;
     #[inline]
     fn mul(self, y: f64) -> Self::Output {
         An(Binop::new(
@@ -442,7 +450,7 @@ where
     X::Outputs: Size<f64>,
     <X::Inputs as Add<U0>>::Output: Size<f64>,
 {
-    type Output = An<Binop<f64, FrameMul<f64, X::Outputs>, Constant<f64, X::Outputs>, X>>;
+    type Output = An<Binop<f64, FrameMul<X::Outputs, f64>, Constant<X::Outputs, f64>, X>>;
     #[inline]
     fn mul(self, y: An<X>) -> Self::Output {
         An(Binop::new(
@@ -462,7 +470,7 @@ where
     X::Outputs: Size<f32>,
     <X::Inputs as Add<U0>>::Output: Size<f32>,
 {
-    type Output = An<Binop<f32, FrameMul<f32, X::Outputs>, X, Constant<f32, X::Outputs>>>;
+    type Output = An<Binop<f32, FrameMul<X::Outputs, f32>, X, Constant<X::Outputs, f32>>>;
     #[inline]
     fn mul(self, y: f32) -> Self::Output {
         An(Binop::new(
@@ -482,7 +490,7 @@ where
     X::Outputs: Size<f32>,
     <X::Inputs as Add<U0>>::Output: Size<f32>,
 {
-    type Output = An<Binop<f32, FrameMul<f32, X::Outputs>, Constant<f32, X::Outputs>, X>>;
+    type Output = An<Binop<f32, FrameMul<X::Outputs, f32>, Constant<X::Outputs, f32>, X>>;
     #[inline]
     fn mul(self, y: An<X>) -> Self::Output {
         An(Binop::new(
@@ -564,49 +572,3 @@ where
         An(Stack::new(self.0, y.0))
     }
 }
-
-pub struct MonoIter<X>
-where
-    X: AudioNode<Outputs = U1>,
-    X::Sample: Float,
-    X::Inputs: Size<X::Sample>,
-{
-    node: X,
-}
-
-impl<X> MonoIter<X>
-where
-    X: AudioNode<Outputs = U1>,
-    X::Sample: Float,
-    X::Inputs: Size<X::Sample>,
-{
-    pub fn new(node: X) -> Self {
-        MonoIter { node }
-    }
-}
-
-impl<X> Iterator for MonoIter<X>
-where
-    X: AudioNode<Outputs = U1>,
-    X::Sample: Float,
-    X::Inputs: Size<X::Sample>,
-{
-    type Item = X::Sample;
-    /// Processes a sample from an all-zeros input.
-    #[inline]
-    fn next(&mut self) -> Option<Self::Item> {
-        Some(self.node.tick(&Frame::default())[0])
-    }
-}
-
-impl<X> An<X>
-where
-    X: AudioNode<Outputs = U1>,
-    X::Sample: Float,
-    X::Inputs: Size<X::Sample>,
-{
-    /// Return node as an iterator that returns mono samples directly.
-    pub fn into_mono_iter(self) -> MonoIter<X> {
-        MonoIter::new(self.0)
-    }
-}
