@@ -182,8 +182,8 @@ where
 /// Tagged constant. Outputs the (scalar) value of the tag.
 /// - Output 0: value
 #[inline]
-pub fn tag(tag: Tag, value: f64) -> An<Tagged<f64>> {
-    An(Tagged::new(tag, value))
+pub fn tag(id: Tag, value: f64) -> An<Tagged<f64>> {
+    An(Tagged::new(id, value))
 }
 
 /// Zero generator.
@@ -212,6 +212,18 @@ pub fn multipass<N: Size<f64>>() -> An<MultiPass<N, f64>> {
     An(MultiPass::new())
 }
 
+/// Timer node. An empty node that presents time as a parameter.
+#[inline]
+pub fn timer(tag: Tag) -> An<Timer<f64>> {
+    An(Timer::new(DEFAULT_SR, tag))
+}
+
+/// Monitor node. Passes through input and retains the latest input as a parameter.
+#[inline]
+pub fn monitor(tag: Tag) -> An<Monitor<f64>> {
+    An(Monitor::new(tag))
+}
+
 /// Mono sink. Input is discarded.
 #[inline]
 pub fn sink() -> An<Sink<U1, f64>> {
@@ -399,9 +411,10 @@ where
     An(Envelope::new(0.002, DEFAULT_SR, f))
 }
 
-/// Control envelope from time-varying, input dependent function `f(t, input)` with `t` in seconds.
+/// Control envelope from time-varying, input dependent function `f(t, x)` with `t` in seconds.
 /// Spaces samples using pseudorandom jittering.
 /// Synonymous with `lfo2`.
+/// - Input 0: x
 /// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
 #[inline]
 pub fn envelope2<E, R>(f: E) -> An<Envelope2<f64, f64, E, R>>
@@ -413,9 +426,10 @@ where
     An(Envelope2::new(0.002, DEFAULT_SR, f))
 }
 
-/// Control envelope from time-varying, input dependent function `f(t, value)` with `t` in seconds.
+/// Control envelope from time-varying, input dependent function `f(t, x)` with `t` in seconds.
 /// Spaces samples using pseudorandom jittering.
 /// Synonymous with `envelope2`.
+/// - Input 0: x
 /// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
 #[inline]
 pub fn lfo2<E, R>(f: E) -> An<Envelope2<f64, f64, E, R>>
@@ -427,6 +441,38 @@ where
     An(Envelope2::new(0.002, DEFAULT_SR, f))
 }
 
+/// Control envelope from time-varying, input dependent function `f(t, x, y)` with `t` in seconds.
+/// Spaces samples using pseudorandom jittering.
+/// Synonymous with `lfo3`.
+/// - Input 0: x
+/// - Input 1: y
+/// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
+#[inline]
+pub fn envelope3<E, R>(f: E) -> An<Envelope3<f64, f64, E, R>>
+where
+    E: Fn(f64, f64, f64) -> R,
+    R: ConstantFrame<Sample = f64>,
+    R::Size: Size<f64>,
+{
+    An(Envelope3::new(0.002, DEFAULT_SR, f))
+}
+
+/// Control envelope from time-varying, input dependent function `f(t, x, y)` with `t` in seconds.
+/// Spaces samples using pseudorandom jittering.
+/// Synonymous with `envelope3`.
+/// - Input 0: x
+/// - Input 1: y
+/// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
+#[inline]
+pub fn lfo3<E, R>(f: E) -> An<Envelope3<f64, f64, E, R>>
+where
+    E: Fn(f64, f64, f64) -> R,
+    R: ConstantFrame<Sample = f64>,
+    R::Size: Size<f64>,
+{
+    An(Envelope3::new(0.002, DEFAULT_SR, f))
+}
+
 /// Maximum Length Sequence noise generator from an `n`-bit sequence.
 /// - Output 0: repeating white noise sequence of only -1 and 1 values.
 #[inline]
@@ -545,14 +591,15 @@ where
 /// # Example
 /// ```
 /// # use fundsp::hacker::*;
-/// let my_sum = map(|i: &Frame<f64, U2>| Frame::<f64, U1>::splat(i[0] + i[1]));
+/// let my_max = map(|i: &Frame<f64, U2>| max(i[0], i[1]));
 /// ```
 #[inline]
 pub fn map<M, I, O>(f: M) -> An<Map<f64, M, I, O>>
 where
-    M: Fn(&Frame<f64, I>) -> Frame<f64, O>,
+    M: Fn(&Frame<f64, I>) -> O,
     I: Size<f64>,
-    O: Size<f64>,
+    O: ConstantFrame<Sample = f64>,
+    O::Size: Size<f64>,
 {
     An(Map::new(f, Routing::Arbitrary))
 }
@@ -909,11 +956,7 @@ pub fn reverb_stereo(
                             f64,
                             Pipe<
                                 f64,
-                                Pipe<
-                                    f64,
-                                    Pipe<f64, Delay<f64>, Lowpole<f64, f64, U1>>,
-                                    DCBlock<f64, f64>,
-                                >,
+                                Pipe<f64, Pipe<f64, Delay<f64>, Fir<f64, U2>>, DCBlock<f64, f64>>,
                                 Binop<
                                     f64,
                                     FrameMul<U1, f64>,
@@ -935,19 +978,70 @@ pub fn reverb_stereo(
     super::prelude::reverb_stereo::<f64, f64>(wet, time)
 }
 
+/// Saw-like discrete summation formula oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: roughness in 0...1 is the attenuation of successive partials.
+/// - Output 0: DSF wave
+pub fn dsf_saw() -> An<Dsf<f64, U2>> {
+    An(Dsf::new(DEFAULT_SR, 1.0, 0.5))
+}
+
+/// Saw-like discrete summation formula oscillator.
+/// Roughness in 0...1 is the attenuation of successive partials.
+/// - Input 0: frequency in Hz
+/// - Output 0: DSF wave
+pub fn dsf_saw_r(roughness: f64) -> An<Dsf<f64, U1>> {
+    An(Dsf::new(DEFAULT_SR, 1.0, roughness))
+}
+
+/// Square-like discrete summation formula oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: roughness in 0...1 is the attenuation of successive partials.
+/// - Output 0: DSF wave
+pub fn dsf_square() -> An<Dsf<f64, U2>> {
+    An(Dsf::new(DEFAULT_SR, 2.0, 0.5))
+}
+
+/// Square-like discrete summation formula oscillator.
+/// Roughness in 0...1 is the attenuation of successive partials.
+/// - Input 0: frequency in Hz
+/// - Output 0: DSF wave
+pub fn dsf_square_r(roughness: f64) -> An<Dsf<f64, U1>> {
+    An(Dsf::new(DEFAULT_SR, 2.0, roughness))
+}
+
+/// Karplus-Strong plucked string oscillator with `frequency` in Hz.
+/// High frequency damping is in 0...1.
+/// - Input 0: string excitation
+/// - Output 0: oscillator output
+pub fn pluck(frequency: f64, gain_per_second: f64, high_frequency_damping: f64) -> An<Pluck<f64>> {
+    An(Pluck::new(
+        DEFAULT_SR,
+        frequency,
+        gain_per_second,
+        high_frequency_damping,
+    ))
+}
+
 /// Saw wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Output 0: saw wave
 #[inline]
 pub fn saw() -> An<WaveSynth<'static, f64, U1>> {
     An(WaveSynth::new(DEFAULT_SR, &SAW_TABLE))
 }
 
 /// Square wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Output 0: square wave
 #[inline]
 pub fn square() -> An<WaveSynth<'static, f64, U1>> {
     An(WaveSynth::new(DEFAULT_SR, &SQUARE_TABLE))
 }
 
 /// Triangle wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Output 0: triangle wave
 #[inline]
 pub fn triangle() -> An<WaveSynth<'static, f64, U1>> {
     An(WaveSynth::new(DEFAULT_SR, &TRIANGLE_TABLE))
