@@ -1,3 +1,4 @@
+#![cfg_attr(docsrs, feature(doc_cfg))]
 //! FunDSP is an audio processing and synthesis library.
 //!
 //! See `README.md` in crate root folder for an overview.
@@ -10,7 +11,8 @@
     clippy::double_neg,
     clippy::needless_range_loop,
     clippy::manual_range_contains,
-    clippy::too_many_arguments
+    clippy::too_many_arguments,
+    clippy::comparison_chain
 )]
 
 #[macro_use]
@@ -244,12 +246,15 @@ pub mod feedback;
 pub mod filter;
 pub mod fir;
 pub mod hacker;
+pub mod hacker32;
 pub mod math;
 pub mod noise;
 pub mod oscillator;
 pub mod oversample;
 pub mod pan;
+pub mod pattern;
 pub mod prelude;
+pub mod scale;
 pub mod sequencer;
 pub mod shape;
 pub mod signal;
