@@ -212,6 +212,18 @@ pub fn multipass<N: Size<T>, T: Float>() -> An<MultiPass<N, T>> {
     An(MultiPass::new())
 }
 
+/// Timer node. An empty node that presents time as a parameter.
+#[inline]
+pub fn timer<T: Float>(tag: Tag) -> An<Timer<T>> {
+    An(Timer::new(DEFAULT_SR, tag))
+}
+
+/// Monitor node. Passes through input and retains the latest input as a parameter.
+#[inline]
+pub fn monitor<T: Float>(tag: Tag) -> An<Monitor<T>> {
+    An(Monitor::new(tag))
+}
+
 /// Mono sink. Input is discarded.
 #[inline]
 pub fn sink<T: Float>() -> An<Sink<U1, T>> {
@@ -441,6 +453,7 @@ where
 /// Control envelope from time-varying, input dependent function `f(t, x)` with `t` in seconds.
 /// Spaces samples using pseudorandom jittering.
 /// Synonymous with `lfo2`.
+/// - Input 0: x
 /// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
 #[inline]
 pub fn envelope2<T, F, E, R>(f: E) -> An<Envelope2<T, F, E, R>>
@@ -462,6 +475,7 @@ where
 /// Control envelope from time-varying, input dependent function `f(t, x)` with `t` in seconds.
 /// Spaces samples using pseudorandom jittering.
 /// Synonymous with `envelope2`.
+/// - Input 0: x
 /// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
 #[inline]
 pub fn lfo2<T, F, E, R>(f: E) -> An<Envelope2<T, F, E, R>>
@@ -476,6 +490,46 @@ where
     An(Envelope2::new(F::from_f64(0.002), DEFAULT_SR, f))
 }
 
+/// Control envelope from time-varying, input dependent function `f(t, x, y)` with `t` in seconds.
+/// Spaces samples using pseudorandom jittering.
+/// Synonymous with `lfo3`.
+/// - Input 0: x
+/// - Input 1: y
+/// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
+#[inline]
+pub fn envelope3<T, F, E, R>(f: E) -> An<Envelope3<T, F, E, R>>
+where
+    T: Float,
+    F: Float,
+    E: Fn(F, F, F) -> R,
+    R: ConstantFrame<Sample = F>,
+    R::Size: Size<F>,
+    R::Size: Size<T>,
+{
+    // Signals containing frequencies no greater than about 20 Hz would be considered control rate.
+    // Therefore, sampling at 500 Hz means these signals are fairly well represented.
+    // While we represent time in double precision internally, it is often okay to use single precision
+    // in envelopes, as local component time typically does not get far from origin.
+    An(Envelope3::new(F::from_f64(0.002), DEFAULT_SR, f))
+}
+
+/// Control envelope from time-varying, input dependent function `f(t, x)` with `t` in seconds.
+/// Spaces samples using pseudorandom jittering.
+/// Synonymous with `envelope3`.
+/// - Output(s): envelope linearly interpolated from samples at 2 ms intervals (average).
+#[inline]
+pub fn lfo3<T, F, E, R>(f: E) -> An<Envelope3<T, F, E, R>>
+where
+    T: Float,
+    F: Float,
+    E: Fn(F, F, F) -> R,
+    R: ConstantFrame<Sample = F>,
+    R::Size: Size<F>,
+    R::Size: Size<T>,
+{
+    An(Envelope3::new(F::from_f64(0.002), DEFAULT_SR, f))
+}
+
 /// Maximum Length Sequence noise generator from an `n`-bit sequence.
 /// - Output 0: repeating white noise sequence of only -1 and 1 values.
 #[inline]
@@ -597,15 +651,16 @@ where
 /// # Example
 /// ```
 /// # use fundsp::prelude::*;
-/// let my_sum = map(|i: &Frame<f64, U2>| Frame::<f64, U1>::splat(i[0] + i[1]));
+/// let my_max = map(|i: &Frame<f64, U2>| max(i[0], i[1]));
 /// ```
 #[inline]
 pub fn map<T, M, I, O>(f: M) -> An<Map<T, M, I, O>>
 where
     T: Float,
-    M: Fn(&Frame<T, I>) -> Frame<T, O>,
+    M: Fn(&Frame<T, I>) -> O,
     I: Size<T>,
-    O: Size<T>,
+    O: ConstantFrame<Sample = T>,
+    O::Size: Size<T>,
 {
     An(Map::new(f, Routing::Arbitrary))
 }
@@ -681,14 +736,14 @@ pub fn follow<T: Float, F: Real, S: ScalarOrPair<Sample = F>>(t: S) -> An<AFollo
 /// Look-ahead limiter with `(attack, release)` times in seconds.
 /// Look-ahead is equal to the attack time.
 #[inline]
-pub fn limiter<T: Float, S: ScalarOrPair<Sample = f64>>(time: S) -> An<Limiter<T, U1, S>> {
+pub fn limiter<T: Real, S: ScalarOrPair<Sample = T>>(time: S) -> An<Limiter<T, U1, S>> {
     An(Limiter::new(DEFAULT_SR, time))
 }
 
 /// Stereo look-ahead limiter with `(attack, release)` times in seconds.
 /// Look-ahead is equal to the attack time.
 #[inline]
-pub fn limiter_stereo<T: Float, S: ScalarOrPair<Sample = f64>>(time: S) -> An<Limiter<T, U2, S>> {
+pub fn limiter_stereo<T: Real, S: ScalarOrPair<Sample = T>>(time: S) -> An<Limiter<T, U2, S>> {
     An(Limiter::new(DEFAULT_SR, time))
 }
 
@@ -1025,7 +1080,7 @@ pub fn reverb_stereo<T, F>(
                             T,
                             Pipe<
                                 T,
-                                Pipe<T, Pipe<T, Delay<T>, Lowpole<T, F, U1>>, DCBlock<T, F>>,
+                                Pipe<T, Pipe<T, Delay<T>, Fir<T, U2>>, DCBlock<T, F>>,
                                 Binop<T, FrameMul<U1, T>, MultiPass<U1, T>, Constant<U1, T>>,
                             >,
                         >,
@@ -1057,7 +1112,7 @@ where
 
     let line = stack::<U32, T, _, _>(|i| {
         delay::<T>(DELAYS[i as usize])
-            >> lowpole_hz::<T, F>(T::new(1600))
+            >> fir((T::from_f32(0.5), T::from_f32(0.5)))
             >> dcblock_hz::<T, F>(F::new(5))
             >> mul(a)
     });
@@ -1071,6 +1126,55 @@ where
         & mul((T::one() - wet, T::one() - wet))
 }
 
+/// Saw-like discrete summation formula oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: roughness in 0...1 is the attenuation of successive partials.
+/// - Output 0: DSF wave
+pub fn dsf_saw<T: Real>() -> An<Dsf<T, U2>> {
+    An(Dsf::new(DEFAULT_SR, T::new(1), T::from_f32(0.5)))
+}
+
+/// Saw-like discrete summation formula oscillator.
+/// Roughness in 0...1 is the attenuation of successive partials.
+/// - Input 0: frequency in Hz
+/// - Output 0: DSF wave
+pub fn dsf_saw_r<T: Real>(roughness: T) -> An<Dsf<T, U1>> {
+    An(Dsf::new(DEFAULT_SR, T::new(1), roughness))
+}
+
+/// Square-like discrete summation formula oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: roughness in 0...1 is the attenuation of successive partials.
+/// - Output 0: DSF wave
+pub fn dsf_square<T: Real>() -> An<Dsf<T, U2>> {
+    An(Dsf::new(DEFAULT_SR, T::new(2), T::from_f32(0.5)))
+}
+
+/// Square-like discrete summation formula oscillator.
+/// Roughness in 0...1 is the attenuation of successive partials.
+/// - Input 0: frequency in Hz
+/// - Output 0: DSF wave
+pub fn dsf_square_r<T: Real>(roughness: T) -> An<Dsf<T, U1>> {
+    An(Dsf::new(DEFAULT_SR, T::new(2), roughness))
+}
+
+/// Karplus-Strong plucked string oscillator with `frequency` in Hz.
+/// High frequency damping is in 0...1.
+/// - Input 0: string excitation
+/// - Output 0: oscillator output
+pub fn pluck<T: Float>(
+    frequency: T,
+    gain_per_second: T,
+    high_frequency_damping: T,
+) -> An<Pluck<T>> {
+    An(Pluck::new(
+        DEFAULT_SR,
+        frequency,
+        gain_per_second,
+        high_frequency_damping,
+    ))
+}
+
 /// Saw wave oscillator.
 /// - Input 0: frequency in Hz
 /// - Output 0: saw wave
