@@ -244,10 +244,10 @@ where
         })
     }
 
-    fn route(&self, input: &SignalFrame, _frequency: f64) -> SignalFrame {
+    fn route(&self, _input: &SignalFrame, _frequency: f64) -> SignalFrame {
         let mut output = new_signal_frame(self.outputs());
         for i in 0..N::USIZE {
-            output[i] = input[0].distort(0.0);
+            output[i] = Signal::Latency(0.0);
         }
         output
     }
@@ -335,9 +335,9 @@ where
         })
     }
 
-    fn route(&self, input: &SignalFrame, _frequency: f64) -> SignalFrame {
+    fn route(&self, _input: &SignalFrame, _frequency: f64) -> SignalFrame {
         let mut output = new_signal_frame(self.outputs());
-        output[0] = input[0].distort(0.0);
+        output[0] = Signal::Latency(0.0);
         output
     }
 }
