@@ -159,8 +159,11 @@ fn test_basic() {
     check_wave(
         dc((110.0, 220.0, 440.0, 880.0)) >> multipass() >> (sink() | -sine() | sink() | sine()),
     );
-    check_wave(dc((110.0, 220.0)) >> declick_s(0.1) + pass() >> (sine() ^ saw()));
-    check_wave(dc((20.0, 40.0)) >> swap() >> pass() * pass() >> (sine() ^ square()));
+    // TODO. The next test fails with declicker present and dsf_square(0.99), why?
+    // It passes if tolerance is increased to 1.0e-8. Is this acceptable because of
+    // declicker process vs. tick - the phase accumulation is slightly different?
+    check_wave(dc((110.0, 220.0)) >> declick_s(0.1) + pass() >> (saw() ^ dsf_square_r(0.9)));
+    check_wave(dc((20.0, 40.0)) >> swap() >> pass() * pass() >> (dsf_saw_r(0.999) ^ square()));
     check_wave(
         dc((880.0, 440.0)) >> pass() - pass() >> branchf::<U2, _, _>(|f| (f - 0.5) * triangle()),
     );
@@ -193,13 +196,15 @@ fn test_basic() {
     check_wave(sequencer);
 
     check_wave((noise() | envelope(|t| spline_noise(1, t * 10.0))) >> panner());
-    check_wave(noise() >> pan(-0.5));
+    check_wave(noise() >> monitor(0) >> pan(-0.5) | timer(1));
     check_wave(tag(0, 5.0) >> lfo2(|t, x| t * x) | tag(1, 1.0));
+    check_wave((tag(0, 5.0) | tag(1, 3.0)) >> lfo3(|t, x, y| t * x * y) | tag(1, 1.0));
 
     // Wave filtering, tick vs. process rendering, node reseting.
     let input = Wave64::render(44100.0, 1.0, &mut (noise() | noise()));
     check_wave_filter(&input, butterpass_hz(1000.0) | lowpole_hz(100.0));
     check_wave_filter(&input, allpole_delay(0.5) | highpole_hz(500.0));
+    check_wave_filter(&input, pluck(60.0, 0.9, 0.8) | pluck(110.0, 0.5, 0.1));
 
     // Constants.
     let mut d = constant(1.0);
